@@ -24,11 +24,10 @@
 #include <gtkmm/button.h>
 #include <gtkmm/image.h>
 #include <obby/user.hpp>
-#include <obby/local_document_info.hpp>
-#include <obby/local_buffer.hpp>
 #include "preferences.hpp"
 #include "historyentry.hpp"
 #include "logview.hpp"
+#include "buffer_def.hpp"
 
 namespace Gobby
 {
@@ -36,25 +35,31 @@ namespace Gobby
 class Chat: public Gtk::VBox
 {
 public:
-	Chat();
+	Chat(Gtk::Window& parent);
 	virtual ~Chat();
 
-//	signal_chat_type chat_event() const;
-
 	// Calls from the window
-	void obby_start(obby::local_buffer& buf);
+	void obby_start(LocalBuffer& buf);
 	void obby_end();
 	void obby_user_join(const obby::user& user);
 	void obby_user_part(const obby::user& user);
-	void obby_document_insert(obby::local_document_info& document);
-	void obby_document_remove(obby::local_document_info& document);
+	void obby_document_insert(LocalDocumentInfo& document);
+	void obby_document_remove(LocalDocumentInfo& document);
 
-	//void obby_message(const obby::user& user, const Glib::ustring& message);
-	//void obby_server_message(const Glib::ustring& message);
 protected:
-	//void add_line(const obby::user& user, const Glib::ustring& message);
 	void on_chat();
 
+#ifdef GTKMM_GEQ_28
+	bool on_focus_in(GdkEventFocus* event);
+	bool on_focus_out(GdkEventFocus* event);
+#endif
+
+	void on_query_failed(const obby::command_query& query);
+	void on_help(const std::string& name, const std::string& desc);
+
+	void on_remove_result(const obby::command_query& query,
+	                      const obby::command_result& result);
+
 	void on_message(const obby::chat::message& message);
 	void on_user_message(const obby::chat::user_message& message);
 	void on_server_message(const obby::chat::server_message& message);
@@ -68,9 +73,8 @@ protected:
 	void recv_system_line(const std::string& line,
 	                      const obby::chat::system_message& message);
 
-	obby::local_buffer* m_buffer;
-
-//	signal_chat_type m_signal_chat;
+	Gtk::Window& m_parent;
+	LocalBuffer* m_buffer;
 
 	Gtk::Image m_img_btn;
 	Gtk::HBox m_box_chat;
@@ -78,6 +82,8 @@ protected:
 	LogView m_log_chat;
 	HistoryEntry m_ent_chat;
 	Gtk::Button m_btn_chat;
+
+	bool m_focus;
 };
 
 }
