@@ -26,13 +26,11 @@
 #include <gtkmm/button.h>
 #include <gtkmm/notebook.h>
 #include <obby/user.hpp>
-#include <obby/document.hpp>
-#include <obby/local_buffer.hpp>
 
 #include "preferences.hpp"
 #include "docwindow.hpp"
 #include "document.hpp"
-#include "mimemap.hpp"
+#include "buffer_def.hpp"
 #include "header.hpp"
 
 namespace Gobby
@@ -71,35 +69,36 @@ public:
 		signal_document_add_type;
 	typedef sigc::signal<void, DocWindow&>
 		signal_document_remove_type;
-	typedef sigc::signal<void, Document&>
+
+	typedef sigc::signal<void, DocWindow&>
 		signal_document_close_request_type;
-	typedef sigc::signal<void, Document&>
+	typedef sigc::signal<void, DocWindow&>
 		signal_document_cursor_moved_type;
-	typedef sigc::signal<void, Document&>
+	typedef sigc::signal<void, DocWindow&>
 		signal_document_content_changed_type;
-	typedef sigc::signal<void, Document&>
+	typedef sigc::signal<void, DocWindow&>
 		signal_document_language_changed_type;
-	typedef sigc::signal<void, Document&>
+
+	typedef sigc::signal<void, DocWindow&>
 		signal_tab_switched_type;
 
 	Folder(Header& header, const Preferences& preferences);
 
-	// Access to the mime map
-	const MimeMap& get_mime_map() const;
-
+#if 0
 	Glib::RefPtr<const Gtk::SourceLanguagesManager>
 		get_lang_manager() const;
+#endif
 
 	// Calls from the window
 	// TODO: Replace the last 5 of these functions by direct signal
 	// connections to obby::local_buffer
-	void obby_start(obby::local_buffer& buf);
+	void obby_start(LocalBuffer& buf);
 	void obby_end();
 	void obby_user_join(const obby::user& user);
 	void obby_user_part(const obby::user& user);
 	void obby_user_colour(const obby::user& user);
-	void obby_document_insert(obby::local_document_info& document);
-	void obby_document_remove(obby::local_document_info& document);
+	void obby_document_insert(LocalDocumentInfo& document);
+	void obby_document_remove(LocalDocumentInfo& document);
 
 	/** Signal which will be emitted if a document has been added to the
 	 * folder.
@@ -128,54 +127,39 @@ public:
 	signal_tab_switched_type
 		tab_switched_event() const;
 protected:
-	/** Parameter type for the internal enable_document_items function.
-	 */
-	enum DocumentItems {
-		/** Disable all document items.
-		 */
-		DOCUMENT_ITEMS_DISABLE_ALL = 0,
-
-		/** Enable only document items which are useful even if the
-		 * user is not subscribed to the document, disable others.
-		 */
-		DOCUMENT_ITEMS_ENABLE_DISCONN = 1,
-
-		/** Enable all document items.
-		 */
-		DOCUMENT_ITEMS_ENABLE_ALL = 2
-	};
-
 	// Overrides
 	virtual void on_switch_page(GtkNotebookPage* page, guint page_num);
 
+	virtual bool on_key_press_event(GdkEventKey* event);
+
 	// Internals
 	void set_tab_colour(DocWindow& win, const Glib::ustring& colour);
-	void enable_document_items(DocumentItems which);
 
 	// Signal handlers
 	void on_language_changed(const Glib::RefPtr<Gtk::SourceLanguage>& language);
 
 	void on_document_subscribe(const obby::user& user,
-	                           obby::local_document_info& info);
+	                           LocalDocumentInfo& info);
 	void on_document_unsubscribe(const obby::user& user,
-	                             obby::local_document_info& info);
+	                             LocalDocumentInfo& info);
 
 	// Called by on_document_subscribe/unsubscribe if the (un)subscribing
 	// user is the local one.
-	void on_self_subscribe(obby::local_document_info& info);
-	void on_self_unsubscribe(obby::local_document_info& info);
+	void on_self_subscribe(LocalDocumentInfo& info);
+	void on_self_unsubscribe(LocalDocumentInfo& info);
 
 	void on_document_modified_changed(DocWindow& window);
-	void on_document_close(Document& document);
+	void on_document_close(DocWindow& window);
 
-	void on_document_cursor_moved(Document& document);
+	void on_document_cursor_moved(DocWindow& window);
 	void on_document_content_changed(DocWindow& window);
-	void on_document_language_changed(Document& document);
+	void on_document_language_changed(DocWindow& window);
 
 	signal_document_add_type
 		m_signal_document_add;
 	signal_document_remove_type
 		m_signal_document_remove;
+
 	signal_document_close_request_type
 		m_signal_document_close_request;
 	signal_document_cursor_moved_type
@@ -184,6 +168,7 @@ protected:
 		m_signal_document_content_changed;
 	signal_document_language_changed_type
 		m_signal_document_language_changed;
+
 	signal_tab_switched_type
 		m_signal_tab_switched;
 
@@ -201,13 +186,11 @@ protected:
 
 	/** Contains a pointer to the current active obby buffer.
 	 */
-	obby::local_buffer* m_buffer;
+	LocalBuffer* m_buffer;
 
 	/** Connection to the unsubscribe signal.
 	 */
 	sigc::connection m_conn_unsubscribe;
-
-	MimeMap m_mime_map;
 };
 
 }
