@@ -19,12 +19,15 @@
 #ifndef _GOBBY_HEADER_HPP_
 #define _GOBBY_HEADER_HPP_
 
+#include <list>
+
 #include <gtkmm/box.h>
 #include <gtkmm/uimanager.h>
 #include <gtkmm/radioaction.h>
 #include <gtkmm/menubar.h>
 #include <gtkmm/toolbar.h>
 
+#include "application_state.hpp"
 #include "sourceview/sourcelanguage.hpp"
 #include "sourceview/sourcelanguagesmanager.hpp"
 
@@ -46,9 +49,57 @@ public:
 		Code code() const;
 	};
 
+	/** @brief Action that automatically chances sensitivity depending
+	 * on application state.
+	 */
+	class AutoAction
+	{
+	public:
+		typedef Glib::RefPtr<Gtk::Action> action_type;
+
+		AutoAction(action_type action,
+		           const ApplicationState& state,
+		           ApplicationFlags inc_flags,
+			   ApplicationFlags exc_flags);
+
+	protected:
+		void on_state_change(const ApplicationState& state);
+
+		action_type m_action;
+		ApplicationFlags m_inc_flags;
+		ApplicationFlags m_exc_flags;
+	};
+
+	/** @brief Class that stores multiple AutoActions.
+	 *
+	 * Once an AutoAction has been created, it works without any further
+	 * need to access the AutoAction object again, one just needs to
+	 * keep the AutoAction somewhere and release it when the application
+	 * exits.
+	 *
+	 * So this class does just keep auto actions without providing access
+	 * to them.
+	 */
+	class AutoList
+	{
+	public:
+		typedef AutoAction::action_type action_type;
+
+		void add(action_type action,
+		         const ApplicationState& state,
+			 ApplicationFlags inc_flags,
+			 ApplicationFlags exc_flags);
+
+		~AutoList();
+
+	protected:
+		std::list<AutoAction*> m_list;
+	};
+
 	class LanguageWrapper
 	{
 	public:
+		//typedef AutoAction<Gtk::RadioAction> Action;
 		typedef Glib::RefPtr<Gtk::RadioAction> Action;
 		typedef Glib::RefPtr<Gtk::SourceLanguage> Language;
 
@@ -62,16 +113,18 @@ public:
 		Language m_language;
 	};
 
-	Header();
+	typedef Glib::RefPtr<Gtk::SourceLanguagesManager> LangManager;
+
+	Header(const ApplicationState& state, const LangManager& lang_mgr);
 
 	// Access to accelerator groups of the ui manager
 	Glib::RefPtr<Gtk::AccelGroup> get_accel_group();
 	Glib::RefPtr<const Gtk::AccelGroup> get_accel_group() const;
 
-	Glib::RefPtr<Gtk::SourceLanguagesManager>
+	/*Glib::RefPtr<Gtk::SourceLanguagesManager>
 		get_lang_manager();
 	Glib::RefPtr<const Gtk::SourceLanguagesManager>
-		get_lang_manager() const;
+		get_lang_manager() const;*/
 
 	// Access to toolbar & menubar
 	Gtk::MenuBar& get_menubar();
@@ -81,7 +134,6 @@ public:
 	const Glib::RefPtr<Gtk::ActionGroup> group_session;
 	const Glib::RefPtr<Gtk::ActionGroup> group_edit;
 	const Glib::RefPtr<Gtk::ActionGroup> group_user;
-	const Glib::RefPtr<Gtk::ActionGroup> group_view;
 	const Glib::RefPtr<Gtk::ActionGroup> group_window;
 	const Glib::RefPtr<Gtk::ActionGroup> group_help;
 
@@ -104,31 +156,40 @@ public:
 	const Glib::RefPtr<Gtk::Action> action_edit_search_replace;
 	const Glib::RefPtr<Gtk::Action> action_edit_goto_line;
 	const Glib::RefPtr<Gtk::Action> action_edit_preferences;
+	const Glib::RefPtr<Gtk::Action> action_edit_document_preferences;
+	const Glib::RefPtr<Gtk::Action> action_edit_syntax;
+	std::list<LanguageWrapper> action_edit_syntax_languages;
 
 	const Glib::RefPtr<Gtk::Action> action_user;
 	const Glib::RefPtr<Gtk::Action> action_user_set_password;
 	const Glib::RefPtr<Gtk::Action> action_user_set_colour;
 
-	const Glib::RefPtr<Gtk::Action> action_view;
-	const Glib::RefPtr<Gtk::Action> action_view_preferences;
-	const Glib::RefPtr<Gtk::Action> action_view_syntax;
-	std::list<LanguageWrapper> action_view_syntax_languages;
-
 	const Glib::RefPtr<Gtk::Action> action_window;
 	const Glib::RefPtr<Gtk::ToggleAction> action_window_userlist;
 	const Glib::RefPtr<Gtk::ToggleAction> action_window_documentlist;
+	const Glib::RefPtr<Gtk::ToggleAction> action_window_chat;
 
 	const Glib::RefPtr<Gtk::Action> action_help;
 	const Glib::RefPtr<Gtk::Action> action_help_about;
 
 protected:
+	void set_action_auto(const Glib::RefPtr<Gtk::Action>& action,
+	                     const ApplicationState& state,
+	                     ApplicationFlags inc_flags,
+	                     ApplicationFlags exc_flags);
+
 	const Glib::RefPtr<Gtk::UIManager> m_ui_manager;
-	Glib::RefPtr<Gtk::SourceLanguagesManager> m_lang_manager;
+	//Glib::RefPtr<Gtk::SourceLanguagesManager> m_lang_manager;
 
 	Gtk::MenuBar* m_menubar;
 	Gtk::Toolbar* m_toolbar;
 
 	Gtk::RadioButtonGroup m_lang_group;
+
+	/** @brief List that just stores internally the auto actions to have
+	 * the actions change sensitivity automatically.
+	 */
+	AutoList m_auto_actions;
 };
 
 }
