@@ -24,11 +24,10 @@
 #include <gtkmm/spinbutton.h>
 #include <gtkmm/entry.h>
 #include <gtkmm/colorbutton.h>
-#include "defaultdialog.hpp"
 #include "config.hpp"
 #include "features.hpp"
 #include "colorsel.hpp"
-#ifdef WITH_HOWL
+#ifdef WITH_ZEROCONF
 #include <gtkmm/box.h>
 #include <gtkmm/expander.h>
 #include <gtkmm/treeview.h>
@@ -41,11 +40,11 @@
 namespace Gobby
 {
 
-class JoinDialog : public DefaultDialog
+class JoinDialog: public Gtk::Dialog
 {
 public:
-#ifdef WITH_HOWL
-	class Columns : public Gtk::TreeModel::ColumnRecord
+#ifdef WITH_ZEROCONF
+	class Columns: public Gtk::TreeModel::ColumnRecord
 	{
 	public:
 		Columns();
@@ -56,10 +55,12 @@ public:
 	};
 #endif
 
-#ifndef WITH_HOWL
-	JoinDialog(Gtk::Window& parent, Gobby::Config& config);
+#ifndef WITH_ZEROCONF
+	JoinDialog(Gtk::Window& parent,
+	           Config::ParentEntry& config_entry);
 #else
-	JoinDialog(Gtk::Window& parent, Gobby::Config& config,
+	JoinDialog(Gtk::Window& parent,
+	           Config::ParentEntry& config_entry,
 	           obby::zeroconf* zeroconf);
 #endif
 	virtual ~JoinDialog();
@@ -77,16 +78,19 @@ public:
 protected:
 	virtual void on_response(int response_id);
 
-#ifdef WITH_HOWL
+#ifdef WITH_ZEROCONF
 	Gtk::TreeModel::iterator find_entry(const std::string& name) const;
 	virtual bool on_timer();
 	virtual void on_discover(const std::string& name,
 	                         const net6::ipv4_address& addr);
 	virtual void on_leave(const std::string& name);
 	virtual void on_change();
+
+	virtual void on_show();
+	virtual void on_hide();
 #endif
 
-	Gobby::Config& m_config;
+	Gobby::Config::ParentEntry& m_config_entry;
 
 	Gtk::Table m_table;
 	Gtk::Label m_lbl_host;
@@ -96,7 +100,7 @@ protected:
 
 	Gtk::VBox m_vbox;
 
-#ifdef WITH_HOWL
+#ifdef WITH_ZEROCONF
 	Gtk::Expander m_ep_discover;
 	Gtk::TreeView m_session_view;
 	Glib::RefPtr<Gtk::ListStore> m_session_list;
