@@ -23,8 +23,8 @@
 #include <gtkmm/treeview.h>
 #include <gtkmm/treestore.h>
 #include <obby/user.hpp>
-#include <obby/local_buffer.hpp>
 #include "togglewindow.hpp"
+#include "buffer_def.hpp"
 #include "header.hpp"
 
 namespace Gobby
@@ -48,30 +48,30 @@ public:
 	UserList(Gtk::Window& parent,
 	         Header& header,
 		 const Preferences& preferences,
-		 Config::Entry& config_entry);
+		 Config::ParentEntry& config_entry);
 
 	// Calls from the window
 	// TODO: Replace them by signal handlers from buf
-	virtual void obby_start(obby::local_buffer& buf);
+	virtual void obby_start(LocalBuffer& buf);
 	virtual void obby_end();
 	virtual void obby_user_join(const obby::user& user);
 	virtual void obby_user_part(const obby::user& user);
 	virtual void obby_user_colour(const obby::user& user);
-	virtual void obby_document_insert(obby::local_document_info& info);
-	virtual void obby_document_remove(obby::local_document_info& info);
+	virtual void obby_document_insert(LocalDocumentInfo& info);
+	virtual void obby_document_remove(LocalDocumentInfo& info);
 protected:
 	Gtk::TreeIter find_iter(const Gtk::TreeIter& parent,
 	                        const Glib::ustring& text) const;
 	void remove_children(const Gtk::TreeIter& parent);
 
 	void on_user_subscribe(const obby::user& user,
-	                       const obby::local_document_info& info);
+	                       const LocalDocumentInfo& info);
 	void on_user_unsubscribe(const obby::user& user,
-	                         const obby::local_document_info& info);
+	                         const LocalDocumentInfo& info);
 
 	Header& m_header;
 
-	obby::local_buffer* m_buffer;
+	LocalBuffer* m_buffer;
 
 	/** GUI components.
 	 */
