@@ -19,25 +19,34 @@
 #ifndef _GOBBY_WINDOW_HPP_
 #define _GOBBY_WINDOW_HPP_
 
+#include <queue>
+#include <memory>
+
 #include <gtkmm/window.h>
 #include <gtkmm/paned.h>
 #include <gtkmm/frame.h>
 #include <gtkmm/messagedialog.h>
-#include <obby/local_buffer.hpp>
+
 #include "features.hpp"
 #include "icon.hpp"
 #include "config.hpp"
+#include "application_state.hpp"
+#include "ipc.hpp"
 #include "header.hpp"
 #include "docwindow.hpp"
+#include "buffer_def.hpp"
 #include "userlist.hpp"
 #include "documentlist.hpp"
+#include "hostdialog.hpp"
+#include "joindialog.hpp"
 #include "finddialog.hpp"
 #include "gotodialog.hpp"
 #include "folder.hpp"
+#include "document_settings.hpp"
 #include "chat.hpp"
 #include "statusbar.hpp"
 #include "dragdrop.hpp"
-#ifdef WITH_HOWL
+#ifdef WITH_ZEROCONF
 #include <obby/zeroconf.hpp>
 #endif
 
@@ -53,20 +62,49 @@ public:
 	/** Offers a pointer to the currently visible document. If the user
 	 * is not subscribed to a document, NULL is returned.
 	 */
-	Document* get_current_document();
+	DocWindow* get_current_document();
+
+	/** @brief Opens a session with the current default settings.
+	 *
+	 * If initial_dialog is true a dialog to turn the host parameters is
+	 * opened, otherwise the default settings are taken.
+	 *
+	 * If the session opening failed, a dialog appears where the user
+	 * might adjust settings or abort.
+	 *
+	 * This function must not be called when a buffer is already open.
+	 */
+	bool session_open(bool initial_dialog);
+
+	/** @brief Joins a session with the current default settings.
+	 *
+	 * If initial_dialog is true a dialog to turn the join parameters is
+	 * opened, otherwise the default settings are used.
+	 *
+	 * If the session join failed, a dialog appears where the user
+	 * might adjust settings or abort.
+	 *
+	 * This function must not be called when a buffer is already open.
+	 */
+	bool session_join(bool initial_dialog);
 
 	/** Opens a document containing the content of a file mounted on the
 	 * local filesystem.
 	 */
-	void open_local_file(const Glib::ustring& file);
+	void open_local_file(const Glib::ustring& file,
+	                     const std::string& encoding);
 
 	/** Saves an existing document to the given path.
 	 */
-	void save_local_file(Document& doc, const Glib::ustring& file);
+	void save_local_file(DocWindow& doc,
+	                     const Glib::ustring& file,
+	                     const std::string& encoding);
 protected:
 	// Gtk::Window overrides
-	virtual void on_realize();
 	virtual bool on_delete_event(GdkEventAny* event);
+	virtual void on_realize();
+
+	void on_chat_realize();
 
 	// Start/End obby session
 	void obby_start();
@@ -95,11 +133,15 @@ protected:
 	void on_view_preferences();
 	void on_view_language(const Glib::RefPtr<Gtk::SourceLanguage>& lang);
 
+	void on_window_chat();
+
 	// Folder UI handler
 	void on_folder_document_add(DocWindow& window);
 	void on_folder_document_remove(DocWindow& window);
-	void on_folder_document_close_request(Document& document);
-	void on_folder_tab_switched(Document& document);
+	void on_folder_document_close_request(DocWindow& window);
+	void on_folder_tab_switched(DocWindow& window);
+
+	void on_settings_document_insert(const LocalDocumentInfo& info);
 
 	void on_about();
 	void on_quit();
@@ -111,8 +153,11 @@ protected:
 	void on_obby_user_part(const obby::user& user);
 	void on_obby_user_colour(const obby::user& user);
 	void on_obby_user_colour_failed();
-	void on_obby_document_insert(obby::document_info& document);
-	void on_obby_document_remove(obby::document_info& document);
+	void on_obby_document_insert(DocumentInfo& document);
+	void on_obby_document_remove(DocumentInfo& document);
+
+	// IPC signal handlers
+	void on_ipc_file(const std::string& file);
 
 	// Helper functions
 	void apply_preferences();
@@ -121,16 +166,30 @@ protected:
 	void display_error(const Glib::ustring& message,
 	                   const Gtk::MessageType type = Gtk::MESSAGE_ERROR);
 
+	bool session_join_impl(const Glib::ustring& host,
+	                       unsigned int port,
+	                       const Glib::ustring& name,
+	                       const Gdk::Color& color);
+
+	bool session_open_impl(unsigned int port,
+	                       const Glib::ustring& name,
+	                       const Gdk::Color& color,
+	                       const Glib::ustring& password,
+	                       const Glib::ustring& session);
+
 	// Config
 	Config& m_config;
+	Glib::RefPtr<Gtk::SourceLanguagesManager> m_lang_manager;
 	Preferences m_preferences;
 	const IconManager& m_icon_mgr;
 
 	// Paths
 	std::string m_last_path;
-	std::string m_local_file_path;
 	std::string m_prev_session;
 
+	std::string m_local_file_path;
+	std::string m_local_encoding;
+
 	// GUI
 	Gtk::VBox m_mainbox;
 	Gtk::VPaned m_mainpaned;
@@ -138,6 +197,9 @@ protected:
 	Gtk::Frame m_frame_chat;
 	Gtk::Frame m_frame_text;
 
+	ApplicationState m_application_state;
+	DocumentSettings m_document_settings;
+
 	Header m_header;
 	UserList m_userlist;
 	DocumentList m_documentlist;
@@ -149,13 +211,24 @@ protected:
 	Chat m_chat;
 	StatusBar m_statusbar;
 
+	sigc::connection m_conn_chat_realize;
+
 	/** Drag+Drop handler
 	 */
 	std::auto_ptr<DragDrop> m_dnd;
 
+	/** Local IPC handler
+	 */
+	std::auto_ptr<Ipc::LocalInstance> m_ipc;
+	std::queue<std::string> m_file_queue;
+
+	// Dialogs
+	std::auto_ptr<HostDialog> m_host_dlg;
+	std::auto_ptr<JoinDialog> m_join_dlg;
+
 	// obby
-	std::auto_ptr<obby::local_buffer> m_buffer;
-#ifdef WITH_HOWL
+	std::auto_ptr<LocalBuffer> m_buffer;
+#ifdef WITH_ZEROCONF
 	std::auto_ptr<obby::zeroconf> m_zeroconf;
 #endif
 	sigc::connection m_timer_conn;
