@@ -19,6 +19,8 @@
 #include <obby/client_buffer.hpp>
 
 #include <gtkmm/stock.h>
+#include <gtkmm/window.h>
+
 #include <obby/format_string.hpp>
 #include "common.hpp"
 #include "chat.hpp"
@@ -60,11 +62,13 @@ namespace
 
 		func(text.substr(prev) );
 	}
+
 }
 
-Gobby::Chat::Chat()
- : Gtk::VBox(), m_buffer(NULL),
-   m_img_btn(Gtk::Stock::JUMP_TO, Gtk::ICON_SIZE_BUTTON)
+Gobby::Chat::Chat(Gtk::Window& parent)
+ : Gtk::VBox(), m_parent(parent), m_buffer(NULL),
+   m_img_btn(Gtk::Stock::JUMP_TO, Gtk::ICON_SIZE_BUTTON),
+   m_focus(false)
 {
 	m_btn_chat.set_label(_("Send"));
 	m_btn_chat.set_image(m_img_btn);
@@ -88,18 +92,40 @@ Gobby::Chat::Chat()
 
 	set_spacing(5);
 	set_sensitive(false);
+
+#ifdef GTKMM_GEQ_28
+	m_parent.signal_focus_in_event().connect(
+		sigc::mem_fun(*this, &Chat::on_focus_in)
+	);
+
+	m_parent.signal_focus_out_event().connect(
+		sigc::mem_fun(*this, &Chat::on_focus_out)
+	);
+#endif
 }
 
 Gobby::Chat::~Chat()
 {
 }
 
-/*Gobby::Chat::signal_chat_type Gobby::Chat::chat_event() const
+#ifdef GTKMM_GEQ_28
+// m_parent.has_focus() seems not to work, so we keep track of focus
+// ourselves
+bool Gobby::Chat::on_focus_in(GdkEventFocus* event)
 {
-	return m_signal_chat;
-}*/
+	m_focus = true;
+	m_parent.set_urgency_hint(false);
+	return false;
+}
 
-void Gobby::Chat::obby_start(obby::local_buffer& buf)
+bool Gobby::Chat::on_focus_out(GdkEventFocus* event)
+{
+	m_focus = false;
+	return false;
+}
+#endif
+
+void Gobby::Chat::obby_start(LocalBuffer& buf)
 {
 	m_buffer = &buf;
 
@@ -109,6 +135,18 @@ void Gobby::Chat::obby_start(obby::local_buffer& buf)
 
 	set_sensitive(true);
 
+	buf.get_command_queue().query_failed_event().connect(
+		sigc::mem_fun(*this, &Chat::on_query_failed)
+	);
+
+	buf.get_command_queue().help_event().connect(
+		sigc::mem_fun(*this, &Chat::on_help)
+	);
+
+	buf.get_command_queue().result_event("remove").connect(
+		sigc::mem_fun(*this, &Chat::on_remove_result)
+	);
+
 	const obby::chat& chat = buf.get_chat();
 	chat.message_event().connect(
 		sigc::mem_fun(*this, &Chat::on_message) );
@@ -130,82 +168,20 @@ void Gobby::Chat::obby_end()
 
 void Gobby::Chat::obby_user_join(const obby::user& user)
 {
-	/*if(~user.get_flags() & obby::user::flags::CONNECTED)
-		return;
-
-	obby::format_string str(_("%0% has joined") );
-	str << user.get_name();
-	m_log_chat.log(str.str(), "blue");*/
 }
 
 void Gobby::Chat::obby_user_part(const obby::user& user)
 {
-/*	obby::format_string str(_("%0% has left") );
-	str << user.get_name();
-	m_log_chat.log(str.str(), "blue");*/
 }
 
-void Gobby::Chat::obby_document_insert(obby::local_document_info& document)
+void Gobby::Chat::obby_document_insert(LocalDocumentInfo& document)
 {
 }
 
-void Gobby::Chat::obby_document_remove(obby::local_document_info& document)
+void Gobby::Chat::obby_document_remove(LocalDocumentInfo& document)
 {
 }
 
-#if 0
-void Gobby::Chat::obby_message(const obby::user& user,
-                               const Glib::ustring& message)
-{
-	// Make sure we are not deceived by rogue multi line messages
-	Glib::ustring::size_type prev = 0, pos = 0;
-	while( (pos = message.find('\n', pos)) != Glib::ustring::npos)
-	{
-		add_line(user, message.substr(prev, pos - prev) );
-		prev = ++pos;
-	}
-	add_line(user, message.substr(prev));
-}
-
-void Gobby::Chat::add_line(const obby::user& user, const Glib::ustring& message)
-{
-	const obby::user& self = m_buffer->get_self();
-	Glib::ustring name = self.get_name();
-
-	// Check if we have to highlight the line becasue the user's nick name
-	// was found in the the message.
-	Glib::ustring colour = "black";
-
-	// Only check for highlighting if another user wrote this message
-	if(&self != &user)
-	{
-		Glib::ustring::size_type pos = 0;
-		while( (message.find(name, pos)) != Glib::ustring::npos)
-		{
-			// Check that the found position is not part of another
-			// word ('ck' should not be found in 'luck' and such).
-			if(pos > 0 && Glib::Unicode::isalnum(message[pos - 1]) )
-				{ ++ pos; continue; }
-
-			if(pos + name.length() < message.length() &&
-			   Glib::Unicode::isalnum(message[pos + name.length()]))
-				{ ++ pos; continue; }
-
-			// Found occurence
-			colour = "darkred";
-			break;
-		}
-	}
-
-	m_log_chat.log("<" + user.get_name() + "> " + message, colour);
-}
-
-void Gobby::Chat::obby_server_message(const Glib::ustring& message)
-{
-	m_log_chat.log(message, "forest green");
-}
-#endif
-
 void Gobby::Chat::on_chat()
 {
 	if(m_buffer == NULL)
@@ -215,11 +191,72 @@ void Gobby::Chat::on_chat()
 	if(message.empty() ) return;
 	m_ent_chat.set_text("");
 
-	// Send each line separately
-	each_line(
-		message,
-		sigc::mem_fun(*this, &Gobby::Chat::send_line)
+	// Commands beginning with /
+	if(message[0] == '/')
+	{
+		Glib::ustring::size_type pos = message.find_first_of(" \n\v\t");
+		if(pos != Glib::ustring::npos)
+		{
+			obby::command_query query(
+				message.substr(1, pos - 1),
+				message.substr(pos + 1)
+			);
+
+			m_buffer->send_command(query);
+		}
+		else
+		{
+			obby::command_query query(message.substr(1), "");
+			m_buffer->send_command(query);
+		}
+	}
+	else
+	{
+		// Send each line separately
+		each_line(
+			message,
+			sigc::mem_fun(*this, &Gobby::Chat::send_line)
+		);
+	}
+}
+
+void Gobby::Chat::on_query_failed(const obby::command_query& query)
+{
+	obby::format_string str(
+		_("Command '%0%' not found. Type /help for a list of "
+		  "existing commands.")
 	);
+
+	str << query.get_command();
+	m_log_chat.log(str.str(), "red", std::time(NULL) );
+}
+
+void Gobby::Chat::on_help(const std::string& name, const std::string& desc)
+{
+	obby::format_string str(
+		_("%0%: %1%")
+	);
+
+	str << name << desc;
+	m_log_chat.log(str.str(), "black", std::time(NULL) );
+}
+
+void Gobby::Chat::on_remove_result(const obby::command_query& query,
+                                   const obby::command_result& result)
+{
+	if(result.get_type() != obby::command_result::REPLY) return;
+
+	if(result.get_reply() == "doc_not_found")
+	{
+		obby::format_string str(_("Document %0% does not exist") );
+		str << query.get_paramlist();
+		m_log_chat.log(str.str(), "red", std::time(NULL) );
+	}
+	else if(result.get_reply() == "no_doc_given")
+	{
+		Glib::ustring str(_("Usage: /remove <document>") );
+		m_log_chat.log(str, "red", std::time(NULL) );
+	}
 }
 
 void Gobby::Chat::on_message(const obby::chat::message& message)
@@ -239,6 +276,11 @@ void Gobby::Chat::on_message(const obby::chat::message& message)
 		on_system_message(*system_message);
 	else
 		throw std::logic_error("Gobby::Chat::on_message");
+
+#ifdef GTKMM_GEQ_28
+	if(!m_focus)
+		m_parent.set_urgency_hint(true);
+#endif
 }
 
 void Gobby::Chat::on_user_message(const obby::chat::user_message& message)
@@ -286,8 +328,9 @@ void Gobby::Chat::recv_user_line(const std::string& line,
 {
 	// Check each line for highlighting occurence
 	Glib::ustring colour = "black";
-	if(is_highlighted(line, m_buffer->get_self().get_name()) )
-		colour = "darkred";
+	if(&message.get_user() != &m_buffer->get_self())
+		if(is_highlighted(line, m_buffer->get_self().get_name()) )
+			colour = "darkred";
 
 	m_log_chat.log(message.repr(), colour, message.get_timestamp() );
 }
@@ -303,4 +346,3 @@ void Gobby::Chat::recv_system_line(const std::string& line,
 {
 	m_log_chat.log(message.repr(), "blue", message.get_timestamp() );
 }
-
