@@ -23,6 +23,40 @@
 #include "document.hpp"
 #include "folder.hpp"
 
+namespace
+{
+	class KeyMap: private net6::non_copyable
+	{
+	public:
+		static const unsigned int nval = ~0u;
+
+		KeyMap()
+		{
+			m_keyvals[GDK_0] = 9;
+			m_keyvals[GDK_1] = 0;
+			m_keyvals[GDK_2] = 1;
+			m_keyvals[GDK_3] = 2;
+			m_keyvals[GDK_4] = 3;
+			m_keyvals[GDK_5] = 4;
+			m_keyvals[GDK_6] = 5;
+			m_keyvals[GDK_7] = 6;
+			m_keyvals[GDK_8] = 7;
+			m_keyvals[GDK_9] = 8;
+		}
+
+		unsigned int lookup(guint key) const
+		{
+			map_type::const_iterator iter = m_keyvals.find(key);
+			if(iter == m_keyvals.end() ) return nval;
+			return iter->second;
+		}
+
+	private:
+		typedef std::map<guint, unsigned int> map_type;
+		map_type m_keyvals;
+	};
+}
+
 Gobby::Folder::TabLabel::TabLabel(const Glib::ustring& label)
  : m_image(Gtk::Stock::CLOSE, Gtk::ICON_SIZE_MENU), m_label(label),
    m_modified("")
@@ -80,14 +114,15 @@ Gobby::Folder::TabLabel::close_event()
 
 Gobby::Folder::Folder(Header& header,
                       const Preferences& preferences):
-	Gtk::Notebook(), m_header(header), m_preferences(preferences),
-	m_buffer(NULL), m_block_language(false)
+	Gtk::Notebook(),
+	m_block_language(false), m_header(header), m_preferences(preferences),
+	m_buffer(NULL)
 {
 	set_scrollable(true);
 
 	for(std::list<Header::LanguageWrapper>::const_iterator iter =
-		m_header.action_view_syntax_languages.begin();
-	    iter != m_header.action_view_syntax_languages.end();
+		m_header.action_edit_syntax_languages.begin();
+	    iter != m_header.action_edit_syntax_languages.end();
 	    ++ iter)
 	{
 		iter->get_action()->signal_activate().connect(
@@ -100,28 +135,24 @@ Gobby::Folder::Folder(Header& header,
 			)
 		);
 	}
+
+	add_events(Gdk::KEY_PRESS_MASK);
 }
 
-const Gobby::MimeMap& Gobby::Folder::get_mime_map() const
-{
-	return m_mime_map;
-}
-
+#if 0
 Glib::RefPtr<const Gtk::SourceLanguagesManager>
 Gobby::Folder::get_lang_manager() const
 {
 	return m_header.get_lang_manager();
 }
+#endif
 
-void Gobby::Folder::obby_start(obby::local_buffer& buf)
+void Gobby::Folder::obby_start(LocalBuffer& buf)
 {
 	// Remove existing pages from older session
 	while(get_n_pages() )
 		remove_page(0);
 
-	// Disable all document items, there are no documents upon start
-	enable_document_items(DOCUMENT_ITEMS_DISABLE_ALL);
-
 	set_sensitive(true);
 	m_buffer = &buf;
 }
@@ -132,37 +163,20 @@ void Gobby::Folder::obby_end()
 
 	// Insensitive just the text editor to allow to scroll and tab between
 	// the documents
-	for(unsigned int i = 0; i < get_n_pages(); ++ i)
-		static_cast<DocWindow*>(
-			get_nth_page(i)
-		)->get_document().set_sensitive(false);
-
-	// Disable some items, but let save and close still active.
-	// Disable all if the last document has been closed.
-	if(get_n_pages() > 0)
-		enable_document_items(DOCUMENT_ITEMS_ENABLE_DISCONN);
+	for(int i = 0; i < get_n_pages(); ++ i)
+		static_cast<DocWindow*>(get_nth_page(i) )->disable();
 }
 
 void Gobby::Folder::obby_user_join(const obby::user& user)
 {
-	// Pass user join event to documents
-	for(unsigned int i = 0; i < get_n_pages(); ++ i)
-		static_cast<DocWindow*>(get_nth_page(i) )->obby_user_join(user);
 }
 
 void Gobby::Folder::obby_user_part(const obby::user& user)
 {
-	// Pass user part event to documents
-	for(unsigned int i = 0; i < get_n_pages(); ++ i)
-		static_cast<DocWindow*>(get_nth_page(i) )->obby_user_part(user);
 }
 
 void Gobby::Folder::obby_user_colour(const obby::user& user)
 {
-	// Pass user colour event to documents
-	for(unsigned int i = 0; i < get_n_pages(); ++ i)
-		static_cast<DocWindow*>(
-			get_nth_page(i) )->obby_user_colour(user);
 }
 
 namespace
@@ -193,7 +207,7 @@ namespace
 	}
 }
 
-void Gobby::Folder::obby_document_insert(obby::local_document_info& document)
+void Gobby::Folder::obby_document_insert(LocalDocumentInfo& document)
 {
 	// Document subscription handling
 	document.subscribe_event().connect(
@@ -211,7 +225,7 @@ void Gobby::Folder::obby_document_insert(obby::local_document_info& document)
 	);
 }
 
-void Gobby::Folder::obby_document_remove(obby::local_document_info& document)
+void Gobby::Folder::obby_document_remove(LocalDocumentInfo& document)
 {
 	// Nothing to do, the unsubscription signal handler has already removed
 	// the page from the notebook.
@@ -268,20 +282,6 @@ void Gobby::Folder::set_tab_colour(DocWindow& win, const Glib::ustring& colour)
 	label.set_use_markup(true);
 }
 
-void Gobby::Folder::enable_document_items(DocumentItems which)
-{
-	m_header.action_session_document_save->
-		set_sensitive(which >= DOCUMENT_ITEMS_ENABLE_DISCONN);
-	m_header.action_session_document_save_as->
-		set_sensitive(which >= DOCUMENT_ITEMS_ENABLE_DISCONN);
-	m_header.action_session_document_close->
-		set_sensitive(which >= DOCUMENT_ITEMS_ENABLE_DISCONN);
-	m_header.action_view_preferences->
-		set_sensitive(which >= DOCUMENT_ITEMS_ENABLE_DISCONN);
-	m_header.action_view_syntax->
-		set_sensitive(which >= DOCUMENT_ITEMS_ENABLE_DISCONN);
-}
-
 void Gobby::Folder::on_language_changed(const Glib::RefPtr<Gtk::SourceLanguage>& language)
 {
 	// TODO: Gobby::Block
@@ -291,11 +291,11 @@ void Gobby::Folder::on_language_changed(const Glib::RefPtr<Gtk::SourceLanguage>&
 	Gtk::Widget* wnd = get_nth_page(get_current_page() );
 
 	// wnd should not be NULL because the language radio items are only
-	// enabled if
+	// enabled if windows are open
 	if(wnd == NULL)
 		throw std::logic_error("Gobby::Folder::on_language_changed");
 
-	static_cast<DocWindow*>(wnd)->get_document().set_language(language);
+	static_cast<DocWindow*>(wnd)->set_language(language);
 	m_block_language = false;
 }
 
@@ -303,40 +303,38 @@ void Gobby::Folder::on_document_modified_changed(DocWindow& window)
 {
 	// Get tab label for this document
 	TabLabel& label = *static_cast<TabLabel*>(get_tab_label(window) );
-	Document& doc = window.get_document();
 	// Show asterisk as the document's title if it has been modified
-	label.set_modified(doc.get_buffer()->get_modified() );
+	label.set_modified(window.get_modified() );
 }
 
-void Gobby::Folder::on_document_close(Document& document)
+void Gobby::Folder::on_document_close(DocWindow& window)
 {
-	m_signal_document_close_request.emit(document);
+	m_signal_document_close_request.emit(window);
 }
 
 void Gobby::Folder::on_document_subscribe(const obby::user& user,
-                                          obby::local_document_info& info)
+                                          LocalDocumentInfo& info)
 {
 	if(&info.get_buffer().get_self() == &user)
 		on_self_subscribe(info);
 }
 
-void Gobby::Folder::on_self_subscribe(obby::local_document_info& info)
+void Gobby::Folder::on_self_subscribe(LocalDocumentInfo& info)
 {
 	// Create new document
 	DocWindow* new_wnd =
-		Gtk::manage(new DocWindow(info, *this, m_preferences) );
-	Document& new_doc = new_wnd->get_document();
+		Gtk::manage(new DocWindow(info, m_preferences) );
 
 	// Watch update signal to emit document_updated signal if a document
 	// has been updated.
-	new_doc.cursor_moved_event().connect(
+	new_wnd->cursor_moved_event().connect(
 		sigc::bind(
 			sigc::mem_fun(*this, &Folder::on_document_cursor_moved),
-			sigc::ref(new_doc)
+			sigc::ref(*new_wnd)
 		)
 	);
 
-	new_doc.content_changed_event().connect(
+	new_wnd->content_changed_event().connect(
 		sigc::bind(
 			sigc::mem_fun(
 				*this,
@@ -346,17 +344,17 @@ void Gobby::Folder::on_self_subscribe(obby::local_document_info& info)
 		)
 	);
 
-	new_doc.language_changed_event().connect(
+	new_wnd->language_changed_event().connect(
 		sigc::bind(
 			sigc::mem_fun(
 				*this,
 				&Folder::on_document_language_changed
 			),
-			sigc::ref(new_doc)
+			sigc::ref(*new_wnd)
 		)
 	);
 
-	new_doc.get_buffer()->signal_modified_changed().connect(
+	new_wnd->get_document().get_buffer()->signal_modified_changed().connect(
 		sigc::bind(
 			sigc::mem_fun(
 				*this,
@@ -368,17 +366,17 @@ void Gobby::Folder::on_self_subscribe(obby::local_document_info& info)
 
 	// Create label for the tab
 	TabLabel* label = Gtk::manage(
-		new TabLabel(escapehtml(info.get_title()))
+		new TabLabel(escapehtml(info.get_suffixed_title()))
 	);
 
 	label->set_use_markup(true);
-	label->set_modified(new_doc.get_modified() );
+	label->set_modified(new_wnd->get_modified() );
 
 	// Connect close event
 	label->close_event().connect(
 		sigc::bind(
 			sigc::mem_fun(*this, &Folder::on_document_close),
-			sigc::ref(new_doc)
+			sigc::ref(*new_wnd)
 		)
 	);
 
@@ -388,28 +386,24 @@ void Gobby::Folder::on_self_subscribe(obby::local_document_info& info)
 	// Show child
 	new_wnd->show_all();
 
-	// Enable document items as there is at least one document we are
-	// subscribed to.
-	enable_document_items(DOCUMENT_ITEMS_ENABLE_ALL);
-
 	// Emit add_document signal
 	m_signal_document_add.emit(*new_wnd);
 }
 
 void Gobby::Folder::on_document_unsubscribe(const obby::user& user,
-                                            obby::local_document_info& info)
+                                            LocalDocumentInfo& info)
 {
 	if(&info.get_buffer().get_self() == &user)
 		on_self_unsubscribe(info);
 }
 
-void Gobby::Folder::on_self_unsubscribe(obby::local_document_info& info)
+void Gobby::Folder::on_self_unsubscribe(LocalDocumentInfo& info)
 {
 	DocWindow* wnd = NULL;
-	for(unsigned int i = 0; i < get_n_pages(); ++ i)
+	for(int i = 0; i < get_n_pages(); ++ i)
 	{
 		DocWindow* cur_wnd = static_cast<DocWindow*>(get_nth_page(i) );
-		if(&cur_wnd->get_document().get_document() == &info)
+		if(&cur_wnd->get_info() == &info)
 		{
 			wnd = cur_wnd;
 			break;
@@ -425,12 +419,6 @@ void Gobby::Folder::on_self_unsubscribe(obby::local_document_info& info)
 	// is Gtk::managed
 	m_signal_document_remove.emit(*wnd);
 
-	if(get_n_pages() == 0)
-	{
-		// Disable document items if there are no more documents left
-		enable_document_items(DOCUMENT_ITEMS_DISABLE_ALL);
-	}
-
 	// The next time the user subscribes the DocWindow will be
 	// recreated
 	m_conn_unsubscribe.disconnect();
@@ -449,16 +437,15 @@ void Gobby::Folder::on_switch_page(GtkNotebookPage* page, guint page_num)
 
 	// However, if the obby session has been closed the statusbar is empty,
 	// there is no need to update anything.
-	Glib::RefPtr<Gtk::SourceLanguage> language =
-		window.get_document().get_language();
+	Glib::RefPtr<Gtk::SourceLanguage> language = window.get_language();
 
 	// Set correct menu item
 	if(!m_block_language)
 	{
 		m_block_language = true;
 		for(std::list<Header::LanguageWrapper>::const_iterator iter =
-			m_header.action_view_syntax_languages.begin();
-		    iter != m_header.action_view_syntax_languages.end();
+			m_header.action_edit_syntax_languages.begin();
+		    iter != m_header.action_edit_syntax_languages.end();
 		    ++ iter)
 		{
 			if(iter->get_language() == language)
@@ -471,7 +458,7 @@ void Gobby::Folder::on_switch_page(GtkNotebookPage* page, guint page_num)
 	if(m_buffer != NULL)
 	{
 		// Another document has been selected: Emit tabswitched
-		m_signal_tab_switched.emit(window.get_document() );
+		m_signal_tab_switched.emit(window);
 	}
 
 	// TODO: We should put flags into the labels which specify if the
@@ -482,12 +469,46 @@ void Gobby::Folder::on_switch_page(GtkNotebookPage* page, guint page_num)
 	set_tab_colour(window, "#000000");
 }
 
-void Gobby::Folder::on_document_cursor_moved(Document& document)
+
+bool Gobby::Folder::on_key_press_event(GdkEventKey* event)
+{
+	static KeyMap keymap;
+
+	if( (event->state & GDK_MOD1_MASK) == GDK_MOD1_MASK)
+	{
+		unsigned int page = keymap.lookup(event->keyval);
+		if(page != KeyMap::nval)
+		{
+			set_current_page(page);
+			return true;
+		}
+	}
+
+	// Is already used by GtkTextTextView...
+	if( (event->state & (GDK_CONTROL_MASK | GDK_MOD1_MASK)) ==
+	    (GDK_CONTROL_MASK | GDK_MOD1_MASK))
+	{
+		int offset = 0;
+		if(event->keyval == GDK_Page_Up) offset = -1;
+		if(event->keyval == GDK_Page_Down) offset = 1;
+
+		if(offset != 0)
+		{
+			int res = get_current_page() + offset + get_n_pages();
+			set_current_page(res % get_n_pages() );
+			return true;
+		}
+	}
+
+	return false;
+}
+
+void Gobby::Folder::on_document_cursor_moved(DocWindow& window)
 {
 	// Update in the currently visible document? Cursor position has moved.
 	Gtk::Widget* wnd = get_nth_page(get_current_page() );
-	if(&static_cast<DocWindow*>(wnd)->get_document() == &document)
-		m_signal_document_cursor_moved.emit(document);
+	if(wnd == &window)
+		m_signal_document_cursor_moved.emit(window);
 }
 
 void Gobby::Folder::on_document_content_changed(DocWindow& window)
@@ -496,7 +517,7 @@ void Gobby::Folder::on_document_content_changed(DocWindow& window)
 	if(wnd == &window)
 	{
 		// Update in the currently visible document? Update statusbar.
-		m_signal_document_content_changed.emit(window.get_document());
+		m_signal_document_content_changed.emit(window);
 	}
 	else
 	{
@@ -506,10 +527,10 @@ void Gobby::Folder::on_document_content_changed(DocWindow& window)
 	}
 }
 
-void Gobby::Folder::on_document_language_changed(Document& document)
+void Gobby::Folder::on_document_language_changed(DocWindow& window)
 {
 	// Update in the currently visible document? Update statusbar.
 	Gtk::Widget* wnd = get_nth_page(get_current_page() );
-	if(&static_cast<DocWindow*>(wnd)->get_document() == &document)
-		m_signal_document_language_changed.emit(document);
+	if(wnd == &window)
+		m_signal_document_language_changed.emit(window);
 }
