@@ -48,22 +48,21 @@ namespace {
 		"      <separator />"
 		"      <menuitem action=\"EditGotoLine\" />"
 		"      <separator />"
+		"      <menuitem action=\"EditDocumentPreferences\" />"
 		"      <menuitem action=\"EditPreferences\" />"
+		"      <separator />"
+		"      <menu action=\"MenuEditSyntax\">"
+		"        <menuitem action=\"EditSyntaxLanguageNone\" />"
+		"      </menu>"
 		"    </menu>"
 		"    <menu action=\"MenuUser\">"
 		"      <menuitem action=\"UserSetPassword\" />"
 		"      <menuitem action=\"UserSetColour\" />"
 		"    </menu>"
-		"    <menu action=\"MenuView\">"
-		"      <menuitem action=\"ViewPreferences\" />"
-		"      <separator />"
-		"      <menu action=\"MenuViewSyntax\">"
-		"        <menuitem action=\"ViewSyntaxLanguageNone\" />"
-		"      </menu>"
-		"    </menu>"
 		"    <menu action=\"MenuWindow\">"
 		"      <menuitem action=\"WindowUserList\" />"
 		"      <menuitem action=\"WindowDocumentList\" />"
+		"      <menuitem action=\"WindowChat\" />"
 		"    </menu>"
 		"    <menu action=\"MenuHelp\">"
 		"      <menuitem action=\"HelpAbout\" />"
@@ -81,6 +80,7 @@ namespace {
 		"    <separator />"
 		"    <toolitem action=\"WindowUserList\" />"
 		"    <toolitem action=\"WindowDocumentList\" />"
+		"    <toolitem action=\"WindowChat\" />"
 		"  </toolbar>"
 		"</ui>";
 
@@ -149,6 +149,48 @@ Gobby::Header::LanguageWrapper::get_language() const
 	return m_language;
 }
 
+Gobby::Header::AutoAction::AutoAction(action_type action,
+                                      const ApplicationState& state,
+                                      ApplicationFlags inc_flags,
+                                      ApplicationFlags exc_flags):
+	m_action(action), m_inc_flags(inc_flags), m_exc_flags(exc_flags)
+{
+	state.state_changed_event().connect(
+		sigc::bind(
+			sigc::hide(
+				sigc::mem_fun(
+					*this,
+					&AutoAction::on_state_change
+				)
+			),
+			sigc::ref(state)
+		)
+	);
+}
+
+void Gobby::Header::AutoAction::on_state_change(const ApplicationState& state)
+{
+	m_action->set_sensitive(state.query(m_inc_flags, m_exc_flags) );
+}
+
+void Gobby::Header::AutoList::add(action_type action,
+                                  const ApplicationState& state,
+                                  ApplicationFlags inc_flags,
+                                  ApplicationFlags exc_flags)
+{
+	m_list.push_back(new AutoAction(action, state, inc_flags, exc_flags) );
+}
+
+Gobby::Header::AutoList::~AutoList()
+{
+	for(std::list<AutoAction*>::iterator iter = m_list.begin();
+	    iter != m_list.end();
+	    ++ iter)
+	{
+		delete *iter;
+	}
+}
+
 Gobby::Header::Error::Error(Code error_code, const Glib::ustring& error_message)
  : Glib::Error(g_quark_from_static_string("GOBBY_HEADER_ERROR"),
                static_cast<int>(error_code), error_message)
@@ -160,16 +202,17 @@ Gobby::Header::Error::Code Gobby::Header::Error::code() const
 	return static_cast<Code>(gobject_->code);
 }
 
-Gobby::Header::Header():
-	group_app(Gtk::ActionGroup::create() ),
-	group_session(Gtk::ActionGroup::create() ),
-	group_edit(Gtk::ActionGroup::create() ),
-	group_user(Gtk::ActionGroup::create() ),
-	group_view(Gtk::ActionGroup::create() ),
-	group_window(Gtk::ActionGroup::create() ),
-	group_help(Gtk::ActionGroup::create() ),
+#include <iostream>
+Gobby::Header::Header(const ApplicationState& state,
+                      const LangManager& lang_mgr):
+	group_app(Gtk::ActionGroup::create("MenuApp") ),
+	group_session(Gtk::ActionGroup::create("MenuSession") ),
+	group_edit(Gtk::ActionGroup::create("MenuEdit") ),
+	group_user(Gtk::ActionGroup::create("MenuUser") ),
+	group_window(Gtk::ActionGroup::create("MenuWindow") ),
+	group_help(Gtk::ActionGroup::create("MenuHelp") ),
 
-	action_app(Gtk::Action::create("MenuApp", "Gobby") ),
+	action_app(Gtk::Action::create("MenuApp", "_Gobby") ),
 	action_app_session_create(
 		Gtk::Action::create(
 			"AppSessionCreate",
@@ -215,7 +258,7 @@ Gobby::Header::Header():
 		)
 	),
 
-	action_session(Gtk::Action::create("MenuSession", _("Session")) ),
+	action_session(Gtk::Action::create("MenuSession", _("_Session")) ),
 
 	action_session_document_create(
 		Gtk::Action::create(
@@ -262,7 +305,7 @@ Gobby::Header::Header():
 		)
 	),
 
-	action_edit(Gtk::Action::create("MenuEdit", _("Edit")) ),
+	action_edit(Gtk::Action::create("MenuEdit", _("_Edit")) ),
 
 	action_edit_search(
 		Gtk::Action::create(
@@ -296,11 +339,23 @@ Gobby::Header::Header():
 			"EditPreferences",
 			Gtk::Stock::PREFERENCES,
 			_("Preferences..."),
-			_("Shows up a dialog to customise Gobby for your needs")
+			_("Displays a dialog to customise Gobby for your needs")
 		)
 	),
 
-	action_user(Gtk::Action::create("MenuUser", _("User")) ),
+	action_edit_document_preferences(
+		Gtk::Action::create(
+			"EditDocumentPreferences",
+			Gtk::Stock::PREFERENCES,
+			_("Document preferences..."),
+			_("Shows a preferences dialog that is just applied to "
+			  "this document")
+		)
+	),
+
+	action_edit_syntax(Gtk::Action::create("MenuEditSyntax", _("Syntax")) ),
+
+	action_user(Gtk::Action::create("MenuUser", _("_User")) ),
 
 	action_user_set_password(
 		Gtk::Action::create(
@@ -320,42 +375,38 @@ Gobby::Header::Header():
 		)
 	),
 
-	action_view(Gtk::Action::create("MenuView", _("View")) ),
-
-	action_view_preferences(
-		Gtk::Action::create(
-			"ViewPreferences",
-			Gtk::Stock::PREFERENCES,
-			_("Document preferences..."),
-			_("Shows a preferences dialog that is just applied to "
-			  "this document")
-		)
-	),
-
-	action_view_syntax(Gtk::Action::create("MenuViewSyntax", _("Syntax")) ),
-
-	action_window(Gtk::Action::create("MenuWindow", _("Window")) ),
+	action_window(Gtk::Action::create("MenuWindow", _("_Window")) ),
 
 	action_window_userlist(
 		Gtk::ToggleAction::create(
 			"WindowUserList",
 			IconManager::STOCK_USERLIST,
 			_("User list"),
-			_("Shows up a list of users that are currently joined")
+			_("Displays a list of users that are currently joined")
 		)
 	),
 
 	action_window_documentlist(
 		Gtk::ToggleAction::create(
 			"WindowDocumentList",
-			Gtk::Stock::EDIT,
+			IconManager::STOCK_DOCLIST,
 			_("Document list"),
-			_("Shows up a list of documents within the "
+			_("Displays a list of documents within the "
 			  "current session")
 		)
 	),
 
-	action_help(Gtk::Action::create("MenuHelp", _("Help")) ),
+	action_window_chat(
+		Gtk::ToggleAction::create(
+			"WindowChat",
+			IconManager::STOCK_CHAT,
+			_("Chat"),
+			_("Displays a chat to talk to other people in "
+			  "the session")
+		)
+	),
+
+	action_help(Gtk::Action::create("MenuHelp", _("_Help")) ),
 
 	action_help_about(
 		Gtk::Action::create(
@@ -366,9 +417,149 @@ Gobby::Header::Header():
 		)
 	),
 
-	m_ui_manager(Gtk::UIManager::create() ),
-	m_lang_manager(Gtk::SourceLanguagesManager::create() )
+	m_ui_manager(Gtk::UIManager::create() )
 {
+	// Assign auto actions
+	set_action_auto(
+		action_app, state,
+		APPLICATION_NONE, APPLICATION_NONE
+	);
+
+	set_action_auto(
+		action_app_session_create, state,
+		APPLICATION_NONE, APPLICATION_SESSION
+	);
+
+	set_action_auto(
+		action_app_session_join, state,
+		APPLICATION_NONE, APPLICATION_SESSION
+	);
+
+	set_action_auto(
+		action_app_session_save, state,
+		APPLICATION_NONE, APPLICATION_INITIAL
+	);
+
+	set_action_auto(
+		action_app_session_quit, state,
+		APPLICATION_SESSION, APPLICATION_NONE
+	);
+
+	set_action_auto(
+		action_app_quit, state,
+		APPLICATION_NONE, APPLICATION_NONE
+	);
+
+	set_action_auto(
+		action_session, state,
+		APPLICATION_NONE, APPLICATION_INITIAL
+	);
+
+	set_action_auto(
+		action_session_document_create, state,
+		APPLICATION_SESSION, APPLICATION_NONE
+	);
+
+	set_action_auto(
+		action_session_document_open, state,
+		APPLICATION_SESSION, APPLICATION_NONE
+	);
+
+	set_action_auto(
+		action_session_document_save, state,
+		APPLICATION_DOCUMENT, APPLICATION_NONE
+	);
+
+	set_action_auto(
+		action_session_document_save_as, state,
+		APPLICATION_DOCUMENT, APPLICATION_NONE
+	);
+
+	set_action_auto(
+		action_session_document_close, state,
+		APPLICATION_DOCUMENT, APPLICATION_NONE
+	);
+
+	set_action_auto(
+		action_edit, state,
+		APPLICATION_NONE, APPLICATION_NONE
+	);
+
+	set_action_auto(
+		action_edit_search, state,
+		APPLICATION_NONE, APPLICATION_NONE
+	);
+
+	set_action_auto(
+		action_edit_search_replace, state,
+		APPLICATION_NONE, APPLICATION_NONE
+	);
+
+	set_action_auto(
+		action_edit_goto_line, state,
+		APPLICATION_NONE, APPLICATION_NONE
+	);
+
+	set_action_auto(
+		action_edit_preferences, state,
+		APPLICATION_NONE, APPLICATION_NONE
+	);
+
+	set_action_auto(
+		action_edit_document_preferences, state,
+		APPLICATION_DOCUMENT, APPLICATION_NONE
+	);
+
+	set_action_auto(
+		action_edit_syntax, state,
+		APPLICATION_DOCUMENT, APPLICATION_NONE
+	);
+
+	set_action_auto(
+		action_user, state,
+		APPLICATION_SESSION, APPLICATION_NONE
+	);
+
+	set_action_auto(
+		action_user_set_password, state,
+		APPLICATION_SESSION, APPLICATION_HOST
+	);
+
+	set_action_auto(
+		action_user_set_colour, state,
+		APPLICATION_SESSION, APPLICATION_NONE
+	);
+
+	set_action_auto(
+		action_window, state,
+		APPLICATION_NONE, APPLICATION_NONE
+	);
+
+	set_action_auto(
+		action_window_userlist, state,
+		APPLICATION_NONE, APPLICATION_NONE
+	);
+
+	set_action_auto(
+		action_window_documentlist, state,
+		APPLICATION_NONE, APPLICATION_NONE
+	);
+
+	set_action_auto(
+		action_window_chat, state,
+		APPLICATION_NONE, APPLICATION_NONE
+	);
+
+	set_action_auto(
+		action_help, state,
+		APPLICATION_NONE, APPLICATION_NONE
+	);
+
+	set_action_auto(
+		action_help_about, state,
+		APPLICATION_NONE, APPLICATION_NONE
+	);
+
 	// Add basic menu
 	m_ui_manager->add_ui_from_string(ui_desc);
 
@@ -389,20 +580,23 @@ Gobby::Header::Header():
 	group_edit->add(action_edit);
 	group_edit->add(action_edit_search);
 	group_edit->add(action_edit_search_replace);
-	group_edit->add(action_edit_goto_line, Gtk::AccelKey("<control>I") );
+	group_edit->add(
+		action_edit_goto_line,
+		Gtk::AccelKey("<control>I", "<Actions>/MenuEdit/EditGotoLine")
+	);
+
 	group_edit->add(action_edit_preferences);
+	group_edit->add(action_edit_document_preferences);
+	group_edit->add(action_edit_syntax);
 
 	group_user->add(action_user);
 	group_user->add(action_user_set_password);
 	group_user->add(action_user_set_colour);
 
-	group_view->add(action_view);
-	group_view->add(action_view_preferences);
-	group_view->add(action_view_syntax);
-
 	group_window->add(action_window);
 	group_window->add(action_window_userlist);
 	group_window->add(action_window_documentlist);
+	group_window->add(action_window_chat);
 
 	group_help->add(action_help);
 	group_help->add(action_help_about);
@@ -411,12 +605,12 @@ Gobby::Header::Header():
 	// Gtk::SourceLanguage::sourcelanguage_class_.init() is called.
 	// See the TODO item in Glib::wrap(GtkSourceLanguage*, bool) in
 	// sourcelanguage.cpp
-	GtkSourceLanguage* lang = NULL;
-	Glib::wrap(lang, false);
+	//GtkSourceLanguage* lang = NULL;
+	//Glib::wrap(lang, false);
 
 	// Get available languages
 	std::list<Glib::RefPtr<Gtk::SourceLanguage> > lang_list =
-		m_lang_manager->get_available_languages();
+		lang_mgr->get_available_languages();
 
 	// Sort languages by name
 	lang_list.sort(&language_sort_callback);
@@ -424,13 +618,13 @@ Gobby::Header::Header():
 	// Add None-Language
 	Glib::RefPtr<Gtk::RadioAction> action = Gtk::RadioAction::create(
 		m_lang_group,
-		"ViewSyntaxLanguageNone",
+		"EditSyntaxLanguageNone",
 		_("None"),
 		_("Unselects the current language")
 	);
 
-	group_view->add(action);
-	action_view_syntax_languages.push_back(
+	group_edit->add(action);
+	action_edit_syntax_languages.push_back(
 		LanguageWrapper(
 			action,
 			Glib::RefPtr<Gtk::SourceLanguage>(NULL)
@@ -449,19 +643,20 @@ Gobby::Header::Header():
 
 		// Build description string
 		obby::format_string str(_("Selects %0% as language") );
-		str << language->get_name().raw();
+		Glib::ustring name = language->get_name();
+		str << name.raw();
 
 		// Add language to action group
 		remove_dangerous_xml(language_xml_name);
 		action = Gtk::RadioAction::create(
 			m_lang_group,
-			"ViewSyntaxLanguage" + language_xml_name,
+			"EditSyntaxLanguage" + language_xml_name,
 			language->get_name(),
 			str.str()
 		);
 
-		group_view->add(action);
-		action_view_syntax_languages.push_back(
+		group_edit->add(action);
+		action_edit_syntax_languages.push_back(
 			LanguageWrapper(action, language)
 		);
 
@@ -469,9 +664,9 @@ Gobby::Header::Header():
 		Glib::ustring xml_desc =
 			"<ui>"
 			"  <menubar name=\"MenuMainBar\">"
-			"    <menu action=\"MenuView\">"
-			"      <menu action=\"MenuViewSyntax\">"
-			"	 <menuitem action=\"ViewSyntaxLanguage"
+			"    <menu action=\"MenuEdit\">"
+			"      <menu action=\"MenuEditSyntax\">"
+			"	 <menuitem action=\"EditSyntaxLanguage"
 				 + language_xml_name + "\" />"
 			"      </menu>"
 			"    </menu>"
@@ -485,7 +680,6 @@ Gobby::Header::Header():
 	m_ui_manager->insert_action_group(group_session);
 	m_ui_manager->insert_action_group(group_edit);
 	m_ui_manager->insert_action_group(group_user);
-	m_ui_manager->insert_action_group(group_view);
 	m_ui_manager->insert_action_group(group_window);
 	m_ui_manager->insert_action_group(group_help);
 
@@ -524,6 +718,7 @@ Glib::RefPtr<const Gtk::AccelGroup> Gobby::Header::get_accel_group() const
 	return m_ui_manager->get_accel_group();
 }
 
+#if 0
 Glib::RefPtr<Gtk::SourceLanguagesManager> Gobby::Header::get_lang_manager()
 {
 	return m_lang_manager;
@@ -534,6 +729,7 @@ Gobby::Header::get_lang_manager() const
 {
 	return m_lang_manager;
 }
+#endif
 
 Gtk::MenuBar& Gobby::Header::get_menubar()
 {
@@ -544,3 +740,12 @@ Gtk::Toolbar& Gobby::Header::get_toolbar()
 {
 	return *m_toolbar;
 }
+
+void Gobby::Header::set_action_auto(const Glib::RefPtr<Gtk::Action>& action,
+                                    const ApplicationState& state,
+                                    ApplicationFlags inc_flags,
+                                    ApplicationFlags exc_flags)
+{
+	m_auto_actions.add(action, state, inc_flags, exc_flags);
+}
+
