@@ -17,7 +17,6 @@
  */
 
 #include <gtkmm/messagedialog.h>
-#include "io/buffer_wrapper.hpp"
 #include "common.hpp"
 #include "hostprogressdialog.hpp"
 
@@ -31,10 +30,10 @@ Gobby::HostProgressDialog::HostProgressDialog(Gtk::Window& parent,
 	m_port(port), m_username(username), m_color(color), m_session(session),
 	m_error("")
 {
-	set_status_text("Generating RSA key...");
+	set_status_text("Opening obby session...");
 }
 
-std::auto_ptr<obby::host_buffer> Gobby::HostProgressDialog::get_buffer()
+std::auto_ptr<Gobby::HostBuffer> Gobby::HostProgressDialog::get_buffer()
 {
 	return m_buffer;
 }
@@ -64,19 +63,23 @@ void Gobby::HostProgressDialog::on_thread(Thread& thread)
 	// Dialog may now be closed
 	unlock(thread);
 
-	std::auto_ptr<obby::host_buffer> buffer; // Resulting obby buffer
+	std::auto_ptr<HostBuffer> buffer; // Resulting obby buffer
 	Glib::ustring error; // Error message
 
 	// Try to open the server.
 	try
 	{
-		// Create buffer, compute RSA key and stuff
+		// Create buffer
 		buffer.reset(
-			new obby::io::host_buffer(
-#ifdef WIN32
-				parent,
-#endif
-				username, obby::colour(red, green, blue)
+			new HostBuffer(
+				username,
+				obby::colour(red, green, blue)
+			)
+		);
+
+		buffer->set_document_template(
+			HostBuffer::document_type::template_type(
+				*buffer
 			)
 		);
 
