@@ -21,7 +21,7 @@
 #include "common.hpp"
 #include "joindialog.hpp"
 
-#ifdef WITH_HOWL
+#ifdef WITH_ZEROCONF
 Gobby::JoinDialog::Columns::Columns()
 {
 	add(name);
@@ -30,22 +30,25 @@ Gobby::JoinDialog::Columns::Columns()
 }
 #endif
 
-#ifndef WITH_HOWL
-Gobby::JoinDialog::JoinDialog(Gtk::Window& parent, Gobby::Config& config)
+#ifndef WITH_ZEROCONF
+Gobby::JoinDialog::JoinDialog(Gtk::Window& parent,
+                              Config::ParentEntry& config_entry):
 #else
-Gobby::JoinDialog::JoinDialog(Gtk::Window& parent, Gobby::Config& config,
-                              obby::zeroconf* zeroconf)
+Gobby::JoinDialog::JoinDialog(Gtk::Window& parent,
+                              Config::ParentEntry& config_entry,
+                              obby::zeroconf* zeroconf):
 #endif
- : DefaultDialog(_("Join obby session"), parent, true, true),
-   m_config(config),
+	Gtk::Dialog(_("Join obby session"), parent, true, true),
+   m_config_entry(config_entry),
    m_table(4, 2),
    m_lbl_host(_("Host:"), Gtk::ALIGN_RIGHT),
    m_lbl_port(_("Port:"), Gtk::ALIGN_RIGHT),
    m_lbl_name(_("Name:"), Gtk::ALIGN_RIGHT),
-   m_lbl_color(_("Colour:"), Gtk::ALIGN_RIGHT)
-#ifdef WITH_HOWL
-   , m_ep_discover(_("Local network")), m_zeroconf(zeroconf)
+   m_lbl_color(_("Colour:"), Gtk::ALIGN_RIGHT),
+#ifdef WITH_ZEROCONF
+   m_ep_discover(_("Local network")), m_zeroconf(zeroconf),
 #endif
+   m_btn_color(config_entry)
 {
 	// TODO: Read default color as random one from tom's color map
 	Gdk::Color default_color;
@@ -53,14 +56,25 @@ Gobby::JoinDialog::JoinDialog(Gtk::Window& parent, Gobby::Config& config,
 	default_color.set_green(0xcccc);
 	default_color.set_blue(0xffff);
 
-	Glib::ustring host =
-		config["session"]["join_host"].get(Glib::ustring("localhost") );
-	unsigned int port =
-		config["session"]["join_port"].get(6522);
-	Glib::ustring name =
-		config["session"]["name"].get(Glib::get_user_name() );
-	Gdk::Color color =
-		config["session"]["color"].get(default_color);
+	Glib::ustring host = config_entry.get_value<Glib::ustring>(
+		"join_host",
+		Glib::ustring("localhost")
+	);
+
+	unsigned int port = config_entry.get_value<unsigned int>(
+		"join_port",
+		6522
+	);
+
+	Glib::ustring name = config_entry.get_value<Glib::ustring>(
+		"name",
+		Glib::get_user_name()
+	);
+
+	Gdk::Color color =  config_entry.get_value<Gdk::Color>(
+		"color",
+		default_color
+	);
 
 	m_ent_host.set_text(host);
 
@@ -71,6 +85,10 @@ Gobby::JoinDialog::JoinDialog(Gtk::Window& parent, Gobby::Config& config,
 	m_ent_name.set_text(name);
 	m_btn_color.set_color(color);
 
+	m_ent_host.set_activates_default(true);
+	m_ent_port.set_activates_default(true);
+	m_ent_name.set_activates_default(true);
+
 	m_table.attach(m_lbl_host, 0, 1, 0, 1,
 		Gtk::SHRINK | Gtk::FILL, Gtk::SHRINK);
 	m_table.attach(m_lbl_port, 0, 1, 1, 2,
@@ -93,8 +111,8 @@ Gobby::JoinDialog::JoinDialog(Gtk::Window& parent, Gobby::Config& config,
 	m_vbox.set_spacing(5);
 	m_vbox.pack_start(m_table);
 
-#ifdef WITH_HOWL
-	if(m_zeroconf)
+#ifdef WITH_ZEROCONF
+	if(m_zeroconf != NULL)
 	{
 		m_session_list = Gtk::ListStore::create(m_session_cols);
 		m_session_view.set_model(m_session_list);
@@ -111,9 +129,6 @@ Gobby::JoinDialog::JoinDialog(Gtk::Window& parent, Gobby::Config& config,
 			sigc::mem_fun(*this, &JoinDialog::on_discover));
 		m_zeroconf->leave_event().connect(
 			sigc::mem_fun(*this, &JoinDialog::on_leave) );
-		m_zeroconf->discover();
-		m_timer_connection = Glib::signal_timeout().connect(
-			sigc::mem_fun(*this, &JoinDialog::on_timer), 400);
 
 		m_vbox.pack_start(m_ep_discover);
 	}
@@ -123,6 +138,7 @@ Gobby::JoinDialog::JoinDialog(Gtk::Window& parent, Gobby::Config& config,
 
 	add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
 	add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
+	set_default_response(Gtk::RESPONSE_OK);
 
 	show_all();
 	set_border_width(10);
@@ -131,10 +147,6 @@ Gobby::JoinDialog::JoinDialog(Gtk::Window& parent, Gobby::Config& config,
 
 Gobby::JoinDialog::~JoinDialog()
 {
-#ifdef WITH_HOWL
-	if(m_timer_connection.connected() )
-		m_timer_connection.disconnect();
-#endif
 }
 
 Glib::ustring Gobby::JoinDialog::get_host() const
@@ -181,16 +193,16 @@ void Gobby::JoinDialog::on_response(int response_id)
 {
 	if(response_id == Gtk::RESPONSE_OK)
 	{
-		m_config["session"]["join_host"].set(get_host() );
-		m_config["session"]["join_port"].set(get_port() );
-		m_config["session"]["name"].set(get_name() );
-		m_config["session"]["color"].set(get_color() );
+		m_config_entry.set_value("join_host", get_host() );
+		m_config_entry.set_value("join_port", get_port() );
+		m_config_entry.set_value("name", get_name() );
+		m_config_entry.set_value("color", get_color() );
 	}
 
 	Gtk::Dialog::on_response(response_id);
 }
 
-#ifdef WITH_HOWL
+#ifdef WITH_ZEROCONF
 Gtk::TreeModel::iterator
 Gobby::JoinDialog::find_entry(const std::string& name) const
 {
@@ -234,6 +246,24 @@ void Gobby::JoinDialog::on_change()
 	m_ent_host.set_text((*iter)[m_session_cols.host]);
 	m_ent_port.set_value((*iter)[m_session_cols.port]);
 }
+
+void Gobby::JoinDialog::on_show()
+{
+	Gtk::Dialog::on_show();
+	if(m_zeroconf != NULL && !m_timer_connection.connected())
+	{
+		m_session_list->clear();
+		m_zeroconf->discover();
+		m_timer_connection = Glib::signal_timeout().connect(
+			sigc::mem_fun(*this, &JoinDialog::on_timer), 400);
+	}
+}
+
+void Gobby::JoinDialog::on_hide()
+{
+	Gtk::Dialog::on_hide();
+	m_timer_connection.disconnect();
+}
 #endif
 
 
