@@ -1,5 +1,5 @@
 /* gobby - A GTKmm driven libobby client
- * Copyright (C) 2005 0x539 dev group
+ * Copyright (C) 2005, 2006 0x539 dev group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -23,22 +23,19 @@
 #include "common.hpp"
 #include "statusbar.hpp"
 
-Gobby::StatusBar::StatusBar(Header& header, const Folder& folder)
- : Frame(),
-   m_header(header),
-   m_language("", Gtk::ALIGN_LEFT),
-   m_connection("Not connected", Gtk::ALIGN_LEFT),
-   m_position("", Gtk::ALIGN_LEFT)
+Gobby::StatusBar::StatusBar(Header& header, const Folder& folder):
+	m_header(header)
 {
-	m_box.pack_start(m_language, Gtk::PACK_SHRINK);
-	m_box.pack_start(m_sep, Gtk::PACK_SHRINK);
-	m_box.pack_start(m_connection, Gtk::PACK_SHRINK);
-	m_box.pack_end(m_position, Gtk::PACK_SHRINK, 2);
-	m_box.set_spacing(5);
+	pack_end(m_bar_position, Gtk::PACK_SHRINK);
+	pack_end(m_bar_language, Gtk::PACK_SHRINK);
 
-	m_sep.hide();
-	add(m_box);
-	set_shadow_type(Gtk::SHADOW_OUT);
+	m_bar_position.set_size_request(200, -1);
+	m_bar_language.set_size_request(200, -1);
+
+	set_has_resize_grip(false);
+	m_bar_language.set_has_resize_grip(false);
+
+	push(_("Not connected"));
 
 	folder.document_cursor_moved_event().connect(
 		sigc::mem_fun(*this, &StatusBar::update_cursor) );
@@ -48,57 +45,57 @@ Gobby::StatusBar::StatusBar(Header& header, const Folder& folder)
 		sigc::mem_fun(*this, &StatusBar::update_from_document) );
 }
 
-Gobby::StatusBar::~StatusBar()
-{
-}
-
-void Gobby::StatusBar::update_language(Document& document)
+void Gobby::StatusBar::update_language(DocWindow& wnd)
 {
 	// Selected language
-	if(document.get_language() )
+	m_bar_language.pop();
+	if(wnd.get_language() )
 	{
 		obby::format_string str(_("Selected language: %0%") );
-		str << document.get_language()->get_name().raw();
-		m_language.set_text(str.str() );
+		str << wnd.get_language()->get_name().raw();
+		m_bar_language.push(str.str() );
 	}
 	else
 	{
-		m_language.set_text(_("No language selected") );
+		m_bar_language.push(_("No language selected") );
 	}
-	m_sep.show();
 }
 
-void Gobby::StatusBar::update_cursor(Document& document)
+void Gobby::StatusBar::update_cursor(DocWindow& wnd)
 {
 	unsigned int row, col;
-	document.get_cursor_position(row, col);
+	wnd.get_cursor_position(row, col);
 
+	m_bar_position.pop();
 	obby::format_string str("Line: %0%, Column: %1%");
 	str << (row + 1) << (col + 1);
-	m_position.set_text(str.str() );
+	m_bar_position.push(str.str() );
 }
 
-void Gobby::StatusBar::update_from_document(Document& document)
+void Gobby::StatusBar::update_from_document(DocWindow& wnd)
 {
-	update_language(document);
-	update_cursor(document);
+	update_language(wnd);
+	update_cursor(wnd);
 }
 
 void Gobby::StatusBar::update_connection(const Glib::ustring& str)
 {
-	m_connection.set_text(str);
+	// TODO: Do this in obby_start!
+	pop();
+	push(str);
 }
 
-void Gobby::StatusBar::obby_start(obby::local_buffer& buf)
+void Gobby::StatusBar::obby_start(LocalBuffer& buf)
 {
 }
 
 void Gobby::StatusBar::obby_end()
 {
-	m_language.set_text("");
-	m_sep.hide();
-	m_connection.set_text("Not connected");
-	m_position.set_text("");
+	pop();
+	m_bar_language.pop();
+	m_bar_position.pop();
+
+	push(_("Not connected"));
 }
 
 void Gobby::StatusBar::obby_user_join(const obby::user& user)
@@ -109,25 +106,24 @@ void Gobby::StatusBar::obby_user_part(const obby::user& user)
 {
 }
 
-void Gobby::StatusBar::obby_document_insert(obby::local_document_info& document)
+void Gobby::StatusBar::obby_document_insert(LocalDocumentInfo& document)
 {
 }
 
-void Gobby::StatusBar::obby_document_remove(obby::local_document_info& document)
+void Gobby::StatusBar::obby_document_remove(LocalDocumentInfo& document)
 {
 	// Last document that is closed?
 	if(document.get_buffer().document_count() == 1)
 	{
 		// Clear document-related statusbar items
-		m_language.set_text("");
-		m_sep.hide();
-		m_position.set_text("");
+		m_bar_language.pop();
+		m_bar_position.pop();
 	}
 }
 
 void Gobby::StatusBar::on_show()
 {
-	Gtk::Frame::on_show();
-	m_sep.hide();
+	Gtk::Statusbar::on_show();
+	//m_sep.hide();
 }
 
