@@ -67,7 +67,7 @@ Gobby::UserList::Columns::Columns()
 Gobby::UserList::UserList(Gtk::Window& parent,
                           Header& header,
                           const Preferences& m_preferences,
-			  Config::Entry& config_entry):
+			  Config::ParentEntry& config_entry):
 	ToggleWindow(
 		parent,
 		header.action_window_userlist,
@@ -120,26 +120,27 @@ Gobby::UserList::UserList(Gtk::Window& parent,
 	show_all_children();
 }
 
-void Gobby::UserList::obby_start(obby::local_buffer& buf)
+void Gobby::UserList::obby_start(LocalBuffer& buf)
 {
-	m_scrolled_wnd.set_sensitive(true);
+	remove_children(m_iter_offline);
+	remove_children(m_iter_online);
 
+	m_scrolled_wnd.set_sensitive(true);
 	m_buffer = &buf;
 }
 
 void Gobby::UserList::obby_end()
 {
-	m_scrolled_wnd.set_sensitive(false);
-
-	remove_children(m_iter_offline);
-	remove_children(m_iter_online);
 }
 
 void Gobby::UserList::obby_user_join(const obby::user& user)
 {
 	// Verify that the user is not already joined
-	if(find_iter(m_iter_online, user.get_name()) != m_iter_online->children().end() )
+	if(find_iter(m_iter_online, user.get_name()) !=
+	   m_iter_online->children().end() )
+	{
 		throw std::logic_error("Gobby::UserList::obby_user_join");
+	}
 
 	// Find user in offline list
 	Gtk::TreeIter iter = find_iter(m_iter_offline, user.get_name() );
@@ -164,7 +165,7 @@ void Gobby::UserList::obby_user_join(const obby::user& user)
 	(*iter)[m_tree_cols.text] = user.get_name();
 
 	// New user may already be subscribed to documents (client initial)
-	for(obby::buffer::document_iterator iter = m_buffer->document_begin();
+	for(Buffer::document_iterator iter = m_buffer->document_begin();
 	    iter != m_buffer->document_end();
 	    ++ iter)
 	{
@@ -172,7 +173,7 @@ void Gobby::UserList::obby_user_join(const obby::user& user)
 		{
 			on_user_subscribe(
 				user,
-				dynamic_cast<const obby::local_document_info&>(
+				dynamic_cast<const LocalDocumentInfo&>(
 					*iter
 				)
 			);
@@ -214,7 +215,7 @@ void Gobby::UserList::obby_user_colour(const obby::user& user)
 	(*iter)[m_tree_cols.icon] = create_coloured_pixbuf(user.get_colour() );
 }
 
-void Gobby::UserList::obby_document_insert(obby::local_document_info& info)
+void Gobby::UserList::obby_document_insert(LocalDocumentInfo& info)
 {
 	info.subscribe_event().connect(
 		sigc::bind(
@@ -231,14 +232,14 @@ void Gobby::UserList::obby_document_insert(obby::local_document_info& info)
 	);
 }
 
-void Gobby::UserList::obby_document_remove(obby::local_document_info& info)
+void Gobby::UserList::obby_document_remove(LocalDocumentInfo& info)
 {
 	// Do nothing here because unsubscrption signal will be emitted for all
 	// users?
 }
 
 void Gobby::UserList::on_user_subscribe(const obby::user& user,
-                                        const obby::local_document_info& info)
+                                        const LocalDocumentInfo& info)
 {
 	Gtk::TreeIter iter = find_iter(m_iter_online, user.get_name() );
 	if(iter == m_iter_online->children().end() )
@@ -254,7 +255,7 @@ void Gobby::UserList::on_user_subscribe(const obby::user& user,
 }
 
 void Gobby::UserList::on_user_unsubscribe(const obby::user& user,
-                                          const obby::local_document_info& info)
+                                          const LocalDocumentInfo& info)
 {
 	Gtk::TreeIter user_iter = find_iter(m_iter_online, user.get_name() );
 	if(user_iter == m_iter_online->children().end() )
